#!/usr/bin/env python
#-*- coding: utf-8 -*-

#===========================================================
# File Name: pdbqt2mol2.py
# Author: wangzhe
# E-mail: wangzhehyd@163.com
# Date: 2018-05-29 09:27:11
# Last Modified: 2018-05-29 21:28:10
# Version: 1.0
# Description: modified from https://github.com/ag83/pdbqt-to-mol2 
# Copyright: Hou group
#===========================================================

import argparse
import sys
import pybel


def usage():
    usage_message = '''This script get coordinates from docked ligand pdbqt file and then replace the corresponding ones in original mol2 file. 
Output a mol2 file with connectivity and properly add hydrogens
Usage: %s
-i original mol2 file
-p original pdbqt file
-r docked pdbqt file
-o output mol2 file
-h Print this help
''' % sys.argv[0]
    print(usage_message)

def get_coord_dict(format, input_mol):
    coord_dict = {}
    mol = pybel.readfile(format, input_mol).next()
    mol.OBMol.DeleteNonPolarHydrogens()
    for atom in mol:
        coord_dict[atom.idx] = atom.coords
    return coord_dict

def coord_map(original_mol2, original_pdbqt, docked_pdbqt):
    if len(original_mol2) == len(original_pdbqt) == len(docked_pdbqt):
        original_mol2_round = {}
        for key in original_mol2:
            coord_update = [round(x, 3) for x in original_mol2[key]]
            coord_update = tuple(coord_update)
            original_mol2_round.update({key:coord_update})

        coord_map = {}
        for idx, coord in original_mol2_round.items():
            for idx2, coord2 in original_pdbqt.items():
                if abs(coord[0] - coord2[0]) < 0.01 and abs(coord[1] - coord2[1]) < 0.01 and abs(coord[2] - coord2[2]) < 0.01:
                    coord_map.update({idx: idx2})
        if len(coord_map) == len(original_mol2):
            return coord_map
        else:
            sys.exit('Lost coordinates in mapping')
    else:
        sys.exit('Not equal number of atoms in molecules')

def convert(original_mol2_mol, docked_pdbqt, coord_map, output_mol2):
    coord_conform = {}
    for idx, idx2 in coord_map.items():
        coord_conform.update({idx:docked_pdbqt.get(idx2)})
    mol2 = pybel.readfile('mol2', original_mol2_mol).next()
    mol2.OBMol.DeleteNonPolarHydrogens()
    for atom in mol2:
        atom.OBAtom.SetVector(coord_conform.get(atom.idx)[0], coord_conform.get(atom.idx)[1], coord_conform.get(atom.idx)[2])
    mol2.OBMol.AddHydrogens()
    mol2.write('mol2', output_mol2, overwrite=True)

def main():
    par_opt = argparse.ArgumentParser()
    par_opt.add_argument('-i', action="store", help='original mol2 file')
    par_opt.add_argument('-l', action="store", help='original pdbqt file')
    par_opt.add_argument('-d', action="store", help='docked pdbqt file')
    par_opt.add_argument('-o', action="store", help='output mol2')
    args = vars(par_opt.parse_args())

    if None in args.values():
        usage()
        sys.exit(2)
    else:
        original_mol2_mol = args['i']
        original_pdbqt_mol = args['l']
        docked_pdbqt_mol = args['d']
        output_mol2 = args['o']
    
    original_mol2 = get_coord_dict('mol2', original_mol2_mol)
    original_pdbqt = get_coord_dict('pdbqt', original_pdbqt_mol)
    docked_pdbqt = get_coord_dict('pdbqt', docked_pdbqt_mol)
    coord_map_dict = coord_map(original_mol2, original_pdbqt, docked_pdbqt)
    convert(original_mol2_mol, docked_pdbqt, coord_map_dict, output_mol2)


if __name__ == '__main__':
    main()
